#include <fxlib.h>
#include "contrast.h"
#include "key.h"
#include "MonochromeLib.h"
#include "functions.h"

FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','C','o','n','t','r','a','s','t','.','d','a','t',0};

unsigned char contrast() {
	unsigned char contrast;
	Tileset *contrastBar, *contrastInit;
	int i;

	contrast = contrast_read();
	if(contrast != 1) {
		set_contrast(contrast);
		return contrast;
	}
		
	contrast = get_contrast();
		
	contrastBar = tileset_load(CONTRAST_BAR);
	contrastInit = tileset_load(CONTRAST_INIT);
	
	do {
		Sleep(50);
		
		if(key_down(K_F1))
			i = 0;
		if((key_down(K_LEFT) || key_down(K_4)) && i > -15)
			i--;
		if((key_down(K_RIGHT) || key_down(K_6)) && i < 15)
			i++;

		set_contrast(contrast + i);
		display_contrast(contrastBar, contrastInit);
	}	
	while(!key_down(K_EXE) && !key_down(K_SHIFT));
	
	save_contrast(contrast + i);
	tileset_free(contrastBar);
	tileset_free(contrastInit);
	
	return contrast + i;
}

void moove_contrast(unsigned char contrastValue, char way) {
	char i;
	for(i = 1; i <= MOOVECONTRASTTIME; i++) {
		set_contrast(contrastValue - MOOVECONTRASTTIME / 2 * (way + 1) + way * i);
		Sleep(80);
	}
}

void set_contrast(unsigned char contrast) // Function from MonochromeLib
{
	char *LCD_register_selector = (char*)0xB4000000, *LCD_data_register = (char*)0xB4010000;
	*LCD_register_selector = 6;
	*LCD_data_register = contrast;
}

static unsigned char get_contrast() // Function from Eiyeron
{
	int model = *(int*)0x80000300;
	
	switch(model) {
		case 0 :
			return 0; // EMULATOR
		break;

		case 0x33363246:
			return 162; // G35
		break;

		case 0x34393542:
			return 168; // G75
		break;

		case 0x80005D7C :
			return 170; // G85
		break;

		case 0x34393342 :
			return 168; // G95
		break;

		default :
			return 165; // UNDEFINED
		break;
	}
}

static void display_contrast(Tileset *contrastBar, Tileset *contrastInit)
{
	ML_bmp_or(contrastBar->tile[0], 32, 26, contrastBar->width, contrastBar->height);
	ML_bmp_or(contrastInit->tile[0], 1, 55, contrastInit->width, contrastInit->height);
	PrintXY(0, 0, "Contrast", 0);
	PrintMini(21, 36, "Light", 0);
	PrintMini(90, 36, "Dark", 0);
	displayVram();
}

static unsigned char contrast_read() // Function from PierrotLL
{
	int file, ret=1;
	file = Bfile_OpenFile(filename, _OPENMODE_READ);
	if(file >= 0) {
		Bfile_ReadFile(file, &ret, 4, 0);
		Bfile_CloseFile(file);
	}
	return ret;
}

static void save_contrast(int contrast) // Function from PierrotLL
{
	int file;
	Bfile_DeleteFile(filename);
	Bfile_CreateFile(filename, 4);
	file = Bfile_OpenFile(filename, _OPENMODE_WRITE);
	if(file >= 0) {
		Bfile_WriteFile(file, &contrast, 4);
		Bfile_CloseFile(file);
	}
}
