#include "game.h"
#include "MonochromeLib.h"
#include "functions.h"

/* 3D engine files */
#include "engine3D.h"
#include "maps.h"
#include "key.h"

/* game files */
#include "weapons.h"
#include "player.h"
#include "robots.h"
#include "menus.h"
#include "contrast.h"
#include "presents.h"

void game(unsigned char contrastValue) {
	unsigned int i;
	
	/* 3D engine var */
	unsigned int currentMap = CURRENTMAP;
	int* map, MapWidth;
	unsigned int timeSinceGetkey = 0;
	unsigned int objectList[nbObject + 1];
	coordonneesCamera camera, cameraMem;
	fpsCount fps = {0};
	collisionDatas collision;
	
	/* game var */
	char begining = 0;
	weapon_datas currentWeapon;
	weapon_name weaponList[NBWEAPON];
	Tileset *bannier, *mancheGun;
	Infos playerInfos = {0};
	robotsInfos robots = {0};
	aim currentAim;
	PresentMovement presentMovement;

	/* 3D engine init */
	MapWidth = mapWidth(currentMap);
	map = (int*)malloc(sizeof(int) * (MapWidth * 8 + 1)); 
	loadMap(map, currentMap, MapWidth, &camera);
	cameraMem = camera;
	loadObjects(&objectList);
	
	/* game init*/
	presentMovement = present_init(presentMovement);
	currentWeapon = weapons_init(currentWeapon, &weaponList);
	playerInfos = infos_init(playerInfos);
	bannier = tileset_load(BANNIER);
	mancheGun = tileset_load(MANCHE);
	robots.nbspawned[0] = 0;
	for(i = 0; i < NBROBOTS; i++)
		robots.list[i] = NOROBOT;
	
	/*
	// Main Loop
	*/

	do {
		char count = 0;
		
		/* 3D engine */
		do {
			timeSinceGetkey = moveCamera(&camera, timeSinceGetkey, currentWeapon.position, &playerInfos);
			collision = view3D(map, &objectList, &camera, &cameraMem, &currentAim, 0);
			count++;
		}	
		while(collision.object > 0 && collision.texture == SOLID && count < 2);
		
		/* game */
		presentMovement = movePresents(map, presentMovement);
		spawnRobots(map, &playerInfos, &robots);
		currentWeapon = setWeapons(currentWeapon, &weaponList, &camera, &playerInfos);
		displayWeapons(currentWeapon);
		playerInfos = getPresent(playerInfos, &weaponList, collision, map, &presentMovement);
		playerInfos = shot(playerInfos, map, &robots, &currentWeapon, currentAim);
		playerInfos = loseLife(playerInfos, robots, camera, contrastValue);
		displayInfos(bannier, mancheGun, &playerInfos);

		//displayFps(&fps);
		displayVram();
		if(begining == 0) {
			moove_contrast(contrastValue, INCREASE);
			begining = 1;
		}
		if(playerInfos.life == 3) {
			tileset_free(bannier);
			tileset_free(mancheGun);
			game_over(playerInfos, contrastValue);
			moove_contrast(contrastValue, DECREASE);
			return;
		}
	}
	while(timeSinceGetkey != -1);
	
	/* free memory */
	tileset_free(bannier);
	tileset_free(mancheGun);
	free(map);
	moove_contrast(contrastValue, DECREASE);
}